from rtde_control import RTDEControlInterface as RTDEControl
import time

rtde_c = RTDEControl("127.0.0.1")

task_frame = [0, 0, 0, 0, 0, 0]
selection_vector = [0, 0, 1, 0, 0, 0]
wrench_down = [0, 0, -10, 0, 0, 0]
wrench_up = [0, 0, 10, 0, 0, 0]
force_type = 2
limits = [2, 2, 1.5, 1, 1, 1]
dt = 1.0/500  # 2ms
joint_q = [-1.54, -1.83, -2.28, -0.59, 1.60, 0.023]

# Move to initial joint position with a regular moveJ
rtde_c.moveJ(joint_q)

# Execute 500Hz control loop for 4 seconds, each cycle is 2ms
for i in range(2000):
    start = time.time()
    # First move the robot down for 2 seconds, then up for 2 seconds
    if i > 1000:
        rtde_c.forceMode(task_frame, selection_vector, wrench_up, force_type, limits)
    else:
        rtde_c.forceMode(task_frame, selection_vector, wrench_down, force_type, limits)
    end = time.time()
    duration = end - start
    if duration < dt:
        time.sleep(dt - duration)

rtde_c.forceModeStop()
rtde_c.stopScript()



from rtde_io import RTDEIOInterface as RTDEIO
from rtde_receive import RTDEReceiveInterface as RTDEReceive
import time

rtde_io_ = RTDEIO("127.0.0.1")
rtde_receive_ = RTDEReceive("127.0.0.1")

# How-to set and get standard and tool digital outputs. Notice that we need the
# RTDEIOInterface for setting an output and RTDEReceiveInterface for getting the state
# of an output.

if rtde_receive_.getDigitalOutState(7):
    print("Standard digital out (7) is HIGH")
else:
    print("Standard digital out (7) is LOW")

if rtde_receive_.getDigitalOutState(16):
    print("Tool digital out (16) is HIGH")
else:
    print("Tool digital out (16) is LOW")

rtde_io_.setStandardDigitalOut(7, True)
rtde_io_.setToolDigitalOut(0, True)
time.sleep(0.01)

if rtde_receive_.getDigitalOutState(7):
    print("Standard digital out (7) is HIGH")
else:
    print("Standard digital out (7) is LOW")

if rtde_receive_.getDigitalOutState(16):
    print("Tool digital out (16) is HIGH")
else:
    print("Tool digital out (16) is LOW")

# How to set a analog output with a specified current ratio
rtde_io_.setAnalogOutputCurrent(1, 0.25)



from rtde_control import RTDEControlInterface as RTDEControl
import time

rtde_c = RTDEControl("127.0.0.1")

# Parameters
acceleration = 0.5
dt = 1.0/500  # 2ms
joint_q = [-1.54, -1.83, -2.28, -0.59, 1.60, 0.023]
joint_speed = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

# Move to initial joint position with a regular moveJ
rtde_c.moveJ(joint_q)

# Execute 500Hz control loop for 2 seconds, each cycle is 2ms
for i in range(1000):
    start = time.time()
    rtde_c.speedJ(joint_speed, acceleration, dt)
    joint_speed[0] += 0.0005
    joint_speed[1] += 0.0005
    end = time.time()
    duration = end - start
    if duration < dt:
        time.sleep(dt - duration)

rtde_c.speedStop()
rtde_c.stopScript()



from rtde_control import RTDEControlInterface as RTDEControl

rtde_c = RTDEControl("127.0.0.1")

velocity = 0.5
acceleration = 0.5
blend_1 = 0.0
blend_2 = 0.02
blend_3 = 0.0
path_pose1 = [-0.143, -0.435, 0.20, -0.001, 3.12, 0.04, velocity, acceleration, blend_1]
path_pose2 = [-0.143, -0.51, 0.21, -0.001, 3.12, 0.04, velocity, acceleration, blend_2]
path_pose3 = [-0.32, -0.61, 0.31, -0.001, 3.12, 0.04, velocity, acceleration, blend_3]
path = [path_pose1, path_pose2, path_pose3]

# Send a linear path with blending in between - (currently uses separate script)
rtde_c.moveL(path)
rtde_c.stopScript()



NEW_LINE = "\n"
QUOTATION = "\""
ROBOTIQ_PREAMBLE = "\n".join([
    "#aliases for the gripper variable names",
    "ACT = 1",
    "GTO = 2",
    "ATR = 3",
    "ARD = 4",
    "FOR = 5",
    "SPE = 6",
    "OBJ = 7",
    "STA = 8",
    "FLT = 9",
    "POS = 10",
    "PRE = 11",
    "LBP = 12",
    "LRD = 13",
    "LBL = 14",
    "LGN = 15",
    "MSC = 16",
    "MOD = 17",
    "",
    "gripper_1_connected = False",
    "gripper_2_connected = False",
    "gripper_3_connected = False",
    "gripper_4_connected = False",
    "",
    "gripper_1_socket_open = False",
    "gripper_2_socket_open = False",
    "gripper_3_socket_open = False",
    "gripper_4_socket_open = False",
    "",
    "def rq_init_comm_if_connected(gripper_sid=9, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    if(not is_gripper_socket_open(gripper_socket)):",
    "      open_gripper_socket(gripper_socket)",
    "    end",
    "",
    "    is_gripper_connected = rq_is_gripper_connected(gripper_sid, gripper_socket)",
    "    if(is_gripper_connected):",
    "	rq_set_gripper_connected(gripper_socket)",
    "    end",
    "",
    "    return is_gripper_connected",
    "end",
    "",
    "def open_gripper_socket(gripper_socket):",
    "  is_open = socket_open(" + QUOTATION + "127.0.0.1" + QUOTATION + ",63352, gripper_socket)",
    "  set_gripper_socket_open(gripper_socket, is_open)",
    "end",
    "",
    "def is_gripper_socket_open(gripper_socket):",
    "  if(gripper_socket == " + QUOTATION + "1" + QUOTATION + "):",
    "    return gripper_1_socket_open",
    "  elif(gripper_socket == " + QUOTATION + "2" + QUOTATION + "):",
    "    return gripper_2_socket_open",
    "  elif(gripper_socket == " + QUOTATION + "3" + QUOTATION + "):",
    "    return gripper_3_socket_open",
    "  elif(gripper_socket == " + QUOTATION + "4" + QUOTATION + "):",
    "    return gripper_4_socket_open",
    "  else:",
    "    return False",
    "  end",
    "end",
    "",
    "def set_gripper_socket_open(gripper_socket, is_open):",
    "  if(gripper_socket == " + QUOTATION + "1" + QUOTATION + "):",
    "    gripper_1_socket_open = is_open",
    "  elif(gripper_socket == " + QUOTATION + "2" + QUOTATION + "):",
    "    gripper_2_socket_open = is_open",
    "  elif(gripper_socket == " + QUOTATION + "3" + QUOTATION + "):",
    "    gripper_3_socket_open = is_open",
    "  elif(gripper_socket == " + QUOTATION + "4" + QUOTATION + "):",
    "    gripper_4_socket_open = is_open",
    "  else:",
    "  end",
    "end",
    "",
    "def rq_is_gripper_connected(gripper_sid=9, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_set_var(" + QUOTATION + "SID" + QUOTATION + ", gripper_sid,  gripper_socket)",
    "    ack = socket_read_byte_list(3, gripper_socket, 0.1)",
    "    return is_ack(ack)",
    "end",
    "",
    "def rq_set_gripper_connected(gripper_id=" + QUOTATION + "1" + QUOTATION + "):",
    "    if(gripper_id == " + QUOTATION + "1" + QUOTATION + "):",
    "	gripper_1_connected = True",
    "    end",
    "",
    "    if (gripper_id == " + QUOTATION + "2" + QUOTATION + "):",
    "	gripper_2_connected = True",
    "    end",
    "",
    "    if (gripper_id == " + QUOTATION + "3" + QUOTATION + "):",
    "	gripper_3_connected = True",
    "    end",
    "",
    "    if (gripper_id == " + QUOTATION + "4" + QUOTATION + "):",
    "	gripper_4_connected = True",
    "    end",
    "end",
    "",
    "def rq_wait_for_gripper_connected():",
    "    gripper_socket = " + QUOTATION + "gripper_conn_socket" + QUOTATION + "",
    "    socket_open(" + QUOTATION + "127.0.0.1" + QUOTATION + ",63352, gripper_socket)",
    "",
    "    retryCtr = 1",
    "    sid_list = rq_get_sid(gripper_socket)",
    "    gripper_is_connected = rq_is_any_gripper_connected(sid_list)",
    "",
    "    while(not gripper_is_connected and retryCtr < 2000):",
    "	retryCtr = retryCtr + 1",
    "	sid_list = rq_get_sid(gripper_socket)",
    "	gripper_is_connected = rq_is_any_gripper_connected(sid_list)",
    "    end",
    "",
    "    socket_close(gripper_socket)",
    "end",
    "",
    "def rq_is_any_gripper_connected(sid_list):",
    "    is_gripper_1_connected = rq_is_gripper1_in_sid_list(sid_list)",
    "    is_gripper_2_connected = rq_is_gripper2_in_sid_list(sid_list)",
    "    is_gripper_3_connected = rq_is_gripper3_in_sid_list(sid_list)",
    "    is_gripper_4_connected = rq_is_gripper4_in_sid_list(sid_list)",
    "",
    "    if(is_gripper_1_connected or is_gripper_2_connected or is_gripper_3_connected or is_gripper_4_connected):",
    "	return True",
    "    else:",
    "	return False",
    "    end",
    "end",
    "",
    "def rq_is_gripper_in_sid_list(gripper_sid, sid_list):",
    "    sid_list_length = sid_list[0]",
    "    sid_list_empty_length = 2",
    "",
    "    if (sid_list_length <= sid_list_empty_length):",
    "	return False",
    "    end",
    "",
    "    sid1 = sid_list[2]",
    "    sid2 = sid_list[5]",
    "    sid3 = sid_list[8]",
    "    sid4 = sid_list[11]",
    "",
    "    if(sid1 == gripper_sid or sid2 == gripper_sid or sid3 == gripper_sid or sid4 == gripper_sid):",
    "	return True",
    "    else:",
    "	return False",
    "    end",
    "end",
    "",
    "def rq_is_gripper1_in_sid_list(sid_list):",
    "    gripper_1_sid_ascii = 57",
    "    return rq_is_gripper_in_sid_list(gripper_1_sid_ascii, sid_list)",
    "end",
    "",
    "def rq_is_gripper2_in_sid_list(sid_list):",
    "    gripper_2_sid_ascii = 50",
    "    return rq_is_gripper_in_sid_list(gripper_2_sid_ascii, sid_list)",
    "end",
    "",
    "def rq_is_gripper3_in_sid_list(sid_list):",
    "    gripper_3_sid_ascii = 51",
    "    return rq_is_gripper_in_sid_list(gripper_3_sid_ascii, sid_list)",
    "end",
    "",
    "def rq_is_gripper4_in_sid_list(sid_list):",
    "    gripper_4_sid_ascii = 52",
    "    return rq_is_gripper_in_sid_list(gripper_4_sid_ascii, sid_list)",
    "end",
    "",
    "def rq_set_sid(gripper_sid=9, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_set_var(" + QUOTATION + "SID" + QUOTATION + ", gripper_sid,  gripper_socket)",
    "    sync()",
    "    ack = socket_read_byte_list(3, gripper_socket)",
    "    return is_ack(ack)",
    "end",
    "",
    "def rq_get_sid(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET SID" + QUOTATION + ", gripper_socket)",
    "    sync()",
    "    sid_list = socket_read_byte_list(17, gripper_socket)",
    "    sync()",
    "    return sid_list",
    "end",
    "",
    "def rq_activate(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_gripper_act = 0",
    "",
    "    if (not rq_is_gripper_activated(gripper_socket)):",
    "       rq_reset(gripper_socket)",
    "",
    "       while(socket_get_var(" + QUOTATION + "ACT" + QUOTATION + ",gripper_socket) == 1):",
    "	  sleep(0.1)",
    "	  rq_reset(gripper_socket)",
    "       end",
    "    end",
    "",
    "    rq_set_var(ACT,1, gripper_socket)",
    "end",
    "",
    "def rq_activate_and_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_activate(gripper_socket)",
    "    sleep(1.0)",
    "",
    "    while(not rq_is_gripper_activated(gripper_socket)):",
    "	# wait for activation completed",
    "    end",
    "    sleep(0.5)",
    "end",
    "",
    "def rq_activate_all_grippers(reset=False):",
    "    if(gripper_1_connected):",
    "	rq_reset_and_activate(" + QUOTATION + "1" + QUOTATION + ", reset)",
    "    end",
    "",
    "    if(gripper_2_connected):",
    "	rq_reset_and_activate(" + QUOTATION + "2" + QUOTATION + ", reset)",
    "    end",
    "",
    "    if(gripper_3_connected):",
    "	rq_reset_and_activate(" + QUOTATION + "3" + QUOTATION + ", reset)",
    "    end",
    "",
    "    if(gripper_4_connected):",
    "	rq_reset_and_activate(" + QUOTATION + "4" + QUOTATION + ", reset)",
    "    end",
    "",
    "    sleep(0.2)",
    "end",
    "",
    "def rq_reset_and_activate(gripper_socket=" + QUOTATION + "1" + QUOTATION + ", reset=False):",
    "    if(reset):",
    "	rq_reset(gripper_socket)",
    "	sleep(0.5)",
    "	rq_activate_and_wait(gripper_socket)",
    "    elif(not rq_is_gripper_activated(gripper_socket)):",
    "	rq_activate_and_wait(gripper_socket)",
    "    end",
    "end",
    "",
    "def rq_scan_block():",
    "    gripper_socket = " + QUOTATION + "scn_block_socket" + QUOTATION + "",
    "    socket_open(" + QUOTATION + "127.0.0.1" + QUOTATION + ", 63352, gripper_socket)",
    "    socket_set_var(" + QUOTATION + "SCN_BLOCK" + QUOTATION + ", 1, gripper_socket)",
    "    sync()",
    "    ack_test = socket_read_byte_list(3, gripper_socket)",
    "",
    "    retry_counter = 0",
    "",
    "    while(not is_ack(ack_test) and retry_counter < 5):",
    "	socket_set_var(" + QUOTATION + "SCN_BLOCK" + QUOTATION + ", 1, gripper_socket)",
    "	sync()",
    "	ack_test = socket_read_byte_list(3, gripper_socket)",
    "	retry_counter = retry_counter + 1",
    "    end",
    "",
    "    socket_close(" + QUOTATION + "scn_block_socket" + QUOTATION + ")",
    "end",
    "",
    "def rq_reset(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_gripper_act = 0",
    "    rq_obj_detect = 0",
    "    rq_mov_complete = 0",
    "",
    "    rq_set_var(ACT,0, gripper_socket)",
    "    rq_set_var(ATR,0, gripper_socket)",
    "end",
    "",
    "def rq_auto_release_open_and_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(ARD,0, gripper_socket)",
    "    rq_set_var(ACT,1, gripper_socket)",
    "    rq_set_var(ATR,0, gripper_socket)",
    "    sleep(0.1)",
    "    rq_set_var(ATR,1, gripper_socket)",
    "",
    "    rq_wait_autorelease_completed(gripper_socket)",
    "end",
    "",
    "def rq_auto_release_close_and_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(ARD,1, gripper_socket)",
    "    rq_set_var(ACT,1, gripper_socket)",
    "    rq_set_var(ATR,0, gripper_socket)",
    "    sleep(0.1)",
    "    rq_set_var(ATR,1, gripper_socket)",
    "",
    "    rq_wait_autorelease_completed(gripper_socket)",
    "end",
    "",
    "def rq_wait_autorelease_completed(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    retryCounter = 1",
    "    gFLT = rq_get_var(FLT, 2, gripper_socket)",
    "",
    "    while(not is_FLT_autorelease_in_progress(gFLT) and retryCounter <= 20):",
    "	retryCounter = retryCounter + 1",
    "	gFLT = rq_get_var(FLT, 2, gripper_socket)",
    "	sleep(0.1)",
    "    end",
    "",
    "    retryCounter = 1",
    "    gFLT = rq_get_var(FLT, 2, gripper_socket)",
    "",
    "    while(not is_FLT_autorelease_completed(gFLT) and retryCounter <= 100):",
    "	retryCounter = retryCounter + 1",
    "	gFLT = rq_get_var(FLT, 2, gripper_socket)",
    "	sleep(0.1)",
    "    end",
    "end",
    "",
    "def rq_set_force(force, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    force = floor(scale(force, [0, 255], [0,255]))",
    "    rq_set_var(FOR, force, gripper_socket)",
    "end",
    "",
    "def rq_set_speed(speed, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    speed = floor(scale(speed, [0, 255], [0,255]))",
    "    rq_set_var(SPE, speed, gripper_socket)",
    "end",
    "",
    "def rq_open(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_move(0, gripper_socket)",
    "end",
    "",
    "def rq_close(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_move(255, gripper_socket)",
    "end",
    "",
    "def rq_open_and_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_move_and_wait(0, gripper_socket)",
    "end",
    "",
    "def rq_close_and_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_move_and_wait(255, gripper_socket)",
    "end",
    "",
    "def rq_move(pos, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_mov_complete = 0",
    "    rq_obj_detect = 0",
    "",
    "    rq_set_pos(pos, gripper_socket)",
    "    rq_go_to(gripper_socket)",
    "end",
    "",
    "def rq_move_and_wait(pos, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_move(pos, gripper_socket)",
    "",
    "    while (not rq_is_motion_complete(gripper_socket)):",
    "	# wait for motion completed",
    "	sleep(0.01)",
    "	sync()",
    "    end",
    "",
    "    # following code used for compatibility with previous versions",
    "    rq_is_object_detected(gripper_socket)",
    "",
    "    if (rq_obj_detect != 1):",
    "	rq_mov_complete = 1",
    "    end",
    "end",
    "",
    "def rq_wait_for_pos_request(pos, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    gPRE = rq_get_var(PRE, 3, gripper_socket)",
    "    pre = (gPRE[1] - 48)*100 + (gPRE[2] -48)*10 + gPRE[3] - 48",
    "",
    "    while (pre != pos):",
    "	rq_set_var(POS, pos, gripper_socket)",
    "	gPRE = rq_get_var(PRE, 3, gripper_socket)",
    "	pre = (gPRE[1] - 48)*100 + (gPRE[2] -48)*10 + gPRE[3] - 48",
    "	sync()",
    "    end",
    "end",
    "",
    "def rq_wait_for_pos(pos, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_wait_for_pos_request(pos, gripper_socket)",
    "",
    "    # Wait for the gripper motion to complete",
    "    while (not rq_is_motion_complete(gripper_socket)):",
    "	# wait for motion completed",
    "	sleep(0.01)",
    "	sync()",
    "	rq_go_to(gripper_socket)",
    "    end",
    "",
    "    # following code used for compatibility with previous versions",
    "    rq_is_object_detected(gripper_socket)",
    "",
    "    if (rq_obj_detect != 1):",
    "	rq_mov_complete = 1",
    "    end",
    "end",
    "",
    "def rq_wait(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    # Wait for the gripper motion to complete",
    "    while (not rq_is_motion_complete(gripper_socket)):",
    "	# wait for motion completed",
    "	sleep(0.01)",
    "	sync()",
    "    end",
    "",
    "    # following code used for compatibility with previous versions",
    "    rq_is_object_detected(gripper_socket)",
    "",
    "    if (rq_obj_detect != 1):",
    "	rq_mov_complete = 1",
    "    end",
    "end",
    "",
    "def rq_wait_for_object_detected(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    # Wait the object detection",
    "    while (not rq_is_object_detected(gripper_socket)):",
    "	# wait for motion completed",
    "	sleep(0.01)",
    "	sync()",
    "    end",
    "end",
    "",
    "# set the position",
    "def rq_set_pos(pos, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos = floor(scale(pos, [0, 255], [0, 255]))",
    "    rq_set_var(POS, pos, gripper_socket)",
    "    rq_wait_for_pos_request(pos, gripper_socket)",
    "end",
    "",
    "# set the position, speed and force",
    "def rq_set_pos_spd_for(pos, speed, force, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    enter_critical",
    "    rq_send_pos_spd_for(pos, speed, force, gripper_socket)",
    "    ack = socket_read_byte_list(3, gripper_socket)",
    "    exit_critical",
    "",
    "    sync()",
    "",
    "    while(is_not_ack(ack)):",
    "	enter_critical",
    "	rq_send_pos_spd_for(pos, speed, force, gripper_socket)",
    "	ack = socket_read_byte_list(3, gripper_socket)",
    "	exit_critical",
    "",
    "	sync()",
    "    end",
    "",
    "    rq_wait_for_pos_request(pos, gripper_socket)",
    "end",
    "",
    "def rq_set_gripper_max_current_mA(current_mA, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    current = floor(current_mA / 10)",
    "    rq_set_var(MSC, current, gripper_socket)",
    "    sleep(1.5)",
    "end",
    "",
    "def rq_set_gripper_mode(mode, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(MOD, mode, gripper_socket)",
    "end",
    "",
    "def rq_set_gripper_max_cur(current_mA, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_gripper_max_current_mA(current_mA, gripper_socket)",
    "end",
    "",
    "def rq_get_gripper_max_current_mA(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET MSC" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    var_value = socket_read_byte_list(3, gripper_socket)",
    "",
    "    current = rq_list_of_bytes_to_value(var_value)",
    "",
    "    if(current == -1):",
    "	current_mA = current",
    "    else:",
    "	current_mA = current * 10",
    "    end",
    "",
    "    return current_mA",
    "end",
    "",
    "def rq_get_gripper_max_cur(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    return rq_get_gripper_max_current_mA(gripper_socket)",
    "end",
    "",
    "def rq_list_of_bytes_to_value(list_of_bytes):",
    "    value = -1",
    "",
    "    # response list length",
    "    if (list_of_bytes[0] == 1):",
    "	value = list_of_bytes[1] - 48",
    "    elif (list_of_bytes[0] == 2):",
    "	value = (list_of_bytes[1] - 48) * 10 + (list_of_bytes[2] - 48)",
    "    elif (list_of_bytes[0] == 3):",
    "	value = (list_of_bytes[1] - 48) * 100 + (list_of_bytes[2] - 48) * 10 + (list_of_bytes[3] - 48)",
    "    end",
    "",
    "    return value",
    "end",
    "",
    "# send the position, speed and force",
    "def rq_send_pos_spd_for(pos, speed, force, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos = floor(scale(pos, [0, 255], [0, 255]))",
    "    speed = floor(scale(speed, [0, 255], [0, 255]))",
    "    force = floor(scale(force, [0, 255], [0, 255]))",
    "",
    "    socket_send_string(" + QUOTATION + "SET POS" + QUOTATION + ", gripper_socket)",
    "    socket_send_byte(32, gripper_socket)",
    "    socket_send_string(pos, gripper_socket)",
    "    socket_send_byte(32, gripper_socket)",
    "    socket_send_string(" + QUOTATION + "SPE" + QUOTATION + ", gripper_socket)",
    "    socket_send_byte(32, gripper_socket)",
    "    socket_send_string(speed, gripper_socket)",
    "    socket_send_byte(32, gripper_socket)",
    "    socket_send_string(" + QUOTATION + "FOR" + QUOTATION + ", gripper_socket)",
    "    socket_send_byte(32, gripper_socket)",
    "    socket_send_string(force, gripper_socket)",
    "    socket_send_byte(10, gripper_socket)",
    "end",
    "",
    "def rq_is_motion_complete(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_mov_complete = 0",
    "",
    "    gOBJ = rq_get_var(OBJ, 1, gripper_socket)",
    "    sleep(0.01)",
    "",
    "    if (is_OBJ_gripper_at_position(gOBJ)):",
    "	rq_mov_complete = 1",
    "	return True",
    "    end",
    "",
    "    if (is_OBJ_object_detected(gOBJ)):",
    "	rq_mov_complete = 1",
    "	return True",
    "    end",
    "",
    "    return False",
    "",
    "end",
    "",
    "def rq_is_gripper_activated(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    gSTA = rq_get_var(STA, 1, gripper_socket)",
    "",
    "    if(is_STA_gripper_activated(gSTA)):",
    "	rq_gripper_act = 1",
    "	return True",
    "    else:",
    "	rq_gripper_act = 0",
    "	return False",
    "    end",
    "end",
    "",
    "def rq_is_object_detected(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    gOBJ = rq_get_var(OBJ, 1, gripper_socket)",
    "",
    "    if(is_OBJ_object_detected(gOBJ)):",
    "	rq_obj_detect = 1",
    "	return True",
    "    else:",
    "	rq_obj_detect = 0",
    "	return False",
    "    end",
    "end",
    "",
    "def rq_current_pos(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    enter_critical",
    "    rq_pos = socket_get_var(" + QUOTATION + "POS" + QUOTATION + ",gripper_socket)",
    "    exit_critical",
    "    sync()",
    "    return rq_pos",
    "end",
    "",
    "def rq_motor_current(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    enter_critical",
    "    rq_current = socket_get_var(" + QUOTATION + "COU" + QUOTATION + ",gripper_socket)",
    "    exit_critical",
    "    sync()",
    "    return rq_current * 10",
    "end",
    "",
    "def rq_print_connected_grippers():",
    "    if(gripper_1_connected):",
    "	textmsg(" + QUOTATION + "Gripper 1 : " + QUOTATION + ", " + QUOTATION + "connected and socket open." + QUOTATION + ")",
    "    end",
    "",
    "    if (gripper_2_connected):",
    "	textmsg(" + QUOTATION + "Gripper 2 : " + QUOTATION + ", " + QUOTATION + "connected and socket open." + QUOTATION + ")",
    "    end",
    "",
    "    if (gripper_3_connected):",
    "	textmsg(" + QUOTATION + "Gripper 3 : " + QUOTATION + ", " + QUOTATION + "connected and socket open." + QUOTATION + ")",
    "    end",
    "",
    "    if (gripper_4_connected):",
    "	textmsg(" + QUOTATION + "Gripper 4 : " + QUOTATION + ", " + QUOTATION + "connected and socket open." + QUOTATION + ")",
    "    end",
    "end",
    "",
    "def rq_print_gripper_fault_code(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    gFLT = rq_get_var(FLT, 2, gripper_socket)",
    "",
    "    if(is_FLT_no_fault(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "No Fault (0x00)" + QUOTATION + ")",
    "    elif (is_FLT_action_delayed(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Priority Fault: Action delayed, initialization must be completed prior to action (0x05)" + QUOTATION + ")",
    "    elif (is_FLT_not_activated(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Priority Fault: The activation must be set prior to action (0x07)" + QUOTATION + ")",
    "    elif (is_FLT_autorelease_in_progress(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Minor Fault: Automatic release in progress (0x0B)" + QUOTATION + ")",
    "    elif (is_FLT_overcurrent(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Minor Fault: Overcurrent protection triggered (0x0E)" + QUOTATION + ")",
    "    elif (is_FLT_autorelease_completed(gFLT)):",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Major Fault: Automatic release completed (0x0F)" + QUOTATION + ")",
    "    else:",
    "	textmsg(" + QUOTATION + "Gripper Fault : " + QUOTATION + ", " + QUOTATION + "Unknown Fault" + QUOTATION + ")",
    "    end",
    "end",
    "",
    "def rq_print_gripper_num_cycles(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET NCY" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "",
    "    if(string_from_server == " + QUOTATION + "0" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Cycle Number : " + QUOTATION + ", " + QUOTATION + "Number of cycles is unreachable." + QUOTATION + ")",
    "    else:",
    "	textmsg(" + QUOTATION + "Gripper Cycle Number : " + QUOTATION + ", string_from_server)",
    "    end",
    "end",
    "",
    "def rq_print_gripper_driver_state(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET DST" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "",
    "    if(string_from_server == " + QUOTATION + "0" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Driver State : " + QUOTATION + ", " + QUOTATION + "RQ_STATE_INIT" + QUOTATION + ")",
    "    elif(string_from_server == " + QUOTATION + "1" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Driver State : " + QUOTATION + ", " + QUOTATION + "RQ_STATE_LISTEN" + QUOTATION + ")",
    "    elif(string_from_server == " + QUOTATION + "2" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Driver State : " + QUOTATION + ", " + QUOTATION + "RQ_STATE_READ_INFO" + QUOTATION + ")",
    "    elif(string_from_server == " + QUOTATION + "3" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Driver State : " + QUOTATION + ", " + QUOTATION + "RQ_STATE_ACTIVATION" + QUOTATION + ")",
    "    else:",
    "	textmsg(" + QUOTATION + "Gripper Driver State : " + QUOTATION + ", " + QUOTATION + "RQ_STATE_RUN" + QUOTATION + ")",
    "    end",
    "end",
    "",
    "def rq_print_gripper_serial_number(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET SNU" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "    textmsg(" + QUOTATION + "Gripper Serial Number : " + QUOTATION + ", string_from_server)",
    "end",
    "",
    "def rq_print_gripper_firmware_version(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET FWV" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "    textmsg(" + QUOTATION + "Gripper Firmware Version : " + QUOTATION + ", string_from_server)",
    "end",
    "",
    "def rq_print_gripper_driver_version(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET VER" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "    textmsg(" + QUOTATION + "Gripper Driver Version : " + QUOTATION + ", string_from_server)",
    "end",
    "",
    "def rq_print_gripper_probleme_connection(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    socket_send_string(" + QUOTATION + "GET PCO" + QUOTATION + ",gripper_socket)",
    "    sync()",
    "    string_from_server = socket_read_string(gripper_socket)",
    "    sync()",
    "    if (string_from_server == " + QUOTATION + "0" + QUOTATION + "):",
    "	textmsg(" + QUOTATION + "Gripper Connection State : " + QUOTATION + ", " + QUOTATION + "No connection problem detected" + QUOTATION + ")",
    "    else:",
    "	textmsg(" + QUOTATION + "Gripper Connection State : " + QUOTATION + ", " + QUOTATION + "Connection problem detected" + QUOTATION + ")",
    "    end",
    "end",
    "",
    "# Returns True if list_of_bytes is [3, 'a', 'c', 'k']",
    "def is_ack(list_of_bytes):",
    "",
    "    # list length is not 3",
    "    if (list_of_bytes[0] != 3):",
    "	return False",
    "    end",
    "",
    "    # first byte not is 'a'?",
    "    if (list_of_bytes[1] != 97):",
    "	return False",
    "    end",
    "",
    "    # first byte not is 'c'?",
    "    if (list_of_bytes[2] != 99):",
    "	return False",
    "    end",
    "",
    "    # first byte not is 'k'?",
    "    if (list_of_bytes[3] != 107):",
    "	return False",
    "    end",
    "",
    "    return True",
    "end",
    "",
    "# Returns True if list_of_bytes is not [3, 'a', 'c', 'k']",
    "def is_not_ack(list_of_bytes):",
    "    if (is_ack(list_of_bytes)):",
    "	return False",
    "    else:",
    "	return True",
    "    end",
    "end",
    "",
    "def is_STA_gripper_activated (list_of_bytes):",
    "",
    "    # list length is not 1",
    "    if (list_of_bytes[0] != 1):",
    "	return False",
    "    end",
    "",
    "    # byte is '3'?",
    "    if (list_of_bytes[1] == 51):",
    "	return True",
    "    end",
    "",
    "    return False",
    "end",
    "",
    "# Returns True if list_of_byte is [1, '1'] or [1, '2']",
    "# Used to test OBJ = 0x1 or OBJ = 0x2",
    "def is_OBJ_object_detected (list_of_bytes):",
    "",
    "    # list length is not 1",
    "    if (list_of_bytes[0] != 1):",
    "	return False",
    "    end",
    "",
    "    # byte is '2'?",
    "    if (list_of_bytes[1] == 50):",
    "	return True",
    "    end",
    "",
    "    # byte is '1'?",
    "    if (list_of_bytes[1]  == 49):",
    "	return True",
    "    end",
    "",
    "    return False",
    "",
    "end",
    "",
    "# Returns True if list_of_byte is [1, '3']",
    "# Used to test OBJ = 0x3",
    "def is_OBJ_gripper_at_position (list_of_bytes):",
    "",
    "    # list length is not 1",
    "    if (list_of_bytes[0] != 1):",
    "	return False",
    "    end",
    "",
    "    # byte is '3'?",
    "    if (list_of_bytes[1] == 51):",
    "	return True",
    "    end",
    "",
    "    return False",
    "end",
    "",
    "def is_not_OBJ_gripper_at_position (list_of_bytes):",
    "",
    "    if (is_OBJ_gripper_at_position(list_of_bytes)):",
    "	return False",
    "    else:",
    "	return True",
    "    end",
    "end",
    "",
    "#### GTO Section ####",
    "def rq_stop(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(GTO, 0, gripper_socket)",
    "end",
    "",
    "def rq_set_GTO_and_wait(value, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(GTO ,value, gripper_socket)",
    "    while(not is_GTO(value, rq_get_var(GTO, 1, gripper_socket))):",
    "      sync()",
    "    end",
    "end",
    "",
    "def rq_go_to(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(GTO, 1, gripper_socket)",
    "end",
    "",
    "",
    "def is_GTO(goto_value, list_of_bytes):",
    "    zero_ascii = 48",
    "    if (list_of_bytes[0] != 1):",
    "	return False",
    "    end",
    "",
    "    if (list_of_bytes[1] == zero_ascii + goto_value):",
    "	return True",
    "    else:",
    "	return False",
    "    end",
    "end",
    "#### GTO Section ####",
    "",
    "def is_FLT_no_fault(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '0'?",
    "    if (list_of_bytes[1] != 48):",
    "	return False",
    "    end",
    "",
    "    # second byte is '0'?",
    "    if (list_of_bytes[2] != 48):",
    "	return False",
    "    end",
    "",
    "    return True",
    "",
    "end",
    "",
    "def is_FLT_action_delayed(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '0'?",
    "    if (list_of_bytes[1] != 48):",
    "	return False",
    "    end",
    "",
    "    # second byte is '5'?",
    "    if (list_of_bytes[2] != 53):",
    "	return False",
    "    end",
    "",
    "    return True",
    "end",
    "",
    "def is_FLT_not_activated(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '0'?",
    "    if (list_of_bytes[1] != 48):",
    "	return False",
    "    end",
    "",
    "    # second byte is '7'?",
    "    if (list_of_bytes[2] != 55):",
    "	return False",
    "    end",
    "",
    "    return True",
    "end",
    "",
    "def is_FLT_autorelease_in_progress(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '1'?",
    "    if (list_of_bytes[1] != 49):",
    "	return False",
    "    end",
    "",
    "    # second byte is '1'?",
    "    if (list_of_bytes[2] != 49):",
    "	return False",
    "    end",
    "",
    "    return True",
    "",
    "end",
    "",
    "def is_FLT_overcurrent(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '1'?",
    "    if (list_of_bytes[1] != 49):",
    "	return False",
    "    end",
    "",
    "    # second byte is '4'?",
    "    if (list_of_bytes[2] != 52):",
    "	return False",
    "    end",
    "",
    "    return True",
    "",
    "end",
    "",
    "def is_FLT_autorelease_completed(list_of_bytes):",
    "",
    "    # list length is not 2",
    "    if (list_of_bytes[0] != 2):",
    "	return False",
    "    end",
    "",
    "    # first byte is '1'?",
    "    if (list_of_bytes[1] != 49):",
    "	return False",
    "    end",
    "",
    "    # second byte is '5'?",
    "    if (list_of_bytes[2] != 53):",
    "	return False",
    "    end",
    "",
    "    return True",
    "",
    "end",
    "",
    "def rq_set_var(var_name, var_value, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "",
    "    var_name_string = " + QUOTATION + "" + QUOTATION + "",
    "",
    "    if (var_name == ACT):",
    "	var_name_string = " + QUOTATION + "ACT" + QUOTATION + "",
    "    elif (var_name == GTO):",
    "	var_name_string = " + QUOTATION + "GTO" + QUOTATION + "",
    "    elif (var_name == ATR):",
    "	var_name_string = " + QUOTATION + "ATR" + QUOTATION + "",
    "    elif (var_name == ARD):",
    "	var_name_string = " + QUOTATION + "ARD" + QUOTATION + "",
    "    elif (var_name == FOR):",
    "	var_name_string = " + QUOTATION + "FOR" + QUOTATION + "",
    "    elif (var_name == SPE):",
    "	var_name_string = " + QUOTATION + "SPE" + QUOTATION + "",
    "    elif (var_name == POS):",
    "	var_name_string = " + QUOTATION + "POS" + QUOTATION + "",
    "    elif (var_name == LBP):",
    "	var_name_string = " + QUOTATION + "LBP" + QUOTATION + "",
    "    elif (var_name == LRD):",
    "	var_name_string = " + QUOTATION + "LRD" + QUOTATION + "",
    "    elif (var_name == LBL):",
    "	var_name_string = " + QUOTATION + "LBL" + QUOTATION + "",
    "    elif (var_name == LGN):",
    "	var_name_string = " + QUOTATION + "LGN" + QUOTATION + "",
    "    elif (var_name == MSC):",
    "	var_name_string = " + QUOTATION + "MSC" + QUOTATION + "",
    "    elif (var_name == MOD):",
    "	var_name_string = " + QUOTATION + "MOD" + QUOTATION + "",
    "    end",
    "",
    "    enter_critical",
    "    socket_set_var(var_name_string, var_value, gripper_socket)",
    "    ack = socket_read_byte_list(3, gripper_socket)",
    "    exit_critical",
    "",
    "    sync()",
    "",
    "    while(is_not_ack(ack)):",
    "	enter_critical",
    "	socket_set_var(var_name_string , var_value, gripper_socket)",
    "	sync()",
    "	ack = socket_read_byte_list(3, gripper_socket)",
    "	exit_critical",
    "",
    "	sync()",
    "    end",
    "end",
    "",
    "",
    "def rq_get_var(var_name, nbr_bytes, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    enter_critical",
    "",
    "    if (var_name == FLT):",
    "	socket_send_string(" + QUOTATION + "GET FLT" + QUOTATION + ", gripper_socket)",
    "    elif (var_name == OBJ):",
    "	socket_send_string(" + QUOTATION + "GET OBJ" + QUOTATION + ", gripper_socket)",
    "    elif (var_name == STA):",
    "	socket_send_string(" + QUOTATION + "GET STA" + QUOTATION + ", gripper_socket)",
    "    elif (var_name == PRE):",
    "	socket_send_string(" + QUOTATION + "GET PRE" + QUOTATION + ", gripper_socket)",
    "    elif (var_name == GTO):",
    "	socket_send_string(" + QUOTATION + "GET GTO" + QUOTATION + ", gripper_socket)",
    "    else:",
    "    end",
    "",
    "    var_value = socket_read_byte_list(nbr_bytes, gripper_socket)",
    "",
    "    exit_critical",
    "",
    "    sync()",
    "",
    "    return var_value",
    "end",
    "",
    "def rq_is_object_validated(gripper_selected, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    if(gripper_selected):",
    "	if(rq_is_object_detected(gripper_socket)):",
    "	    return True",
    "	else:",
    "	    return False",
    "	end",
    "    else:",
    "	return True",
    "    end",
    "end",
    "",
    "############################################",
    "# normalized functions (maps 0-100 to 0-255)",
    "############################################",
    "def rq_set_force_norm(force_norm, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    force_gripper = norm_to_gripper(force_norm)",
    "    rq_set_force(force_gripper, gripper_socket)",
    "end",
    "",
    "def rq_set_speed_norm(speed_norm, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    speed_gripper = norm_to_gripper(speed_norm)",
    "    rq_set_speed(speed_gripper, gripper_socket)",
    "end",
    "",
    "def rq_move_norm(pos_norm, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos_gripper = norm_to_gripper(pos_norm)",
    "    rq_move(pos_gripper, gripper_socket)",
    "end",
    "",
    "def rq_move_and_wait_norm(pos_norm, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos_gripper = norm_to_gripper(pos_norm)",
    "    rq_move_and_wait(pos_gripper, gripper_socket)",
    "end",
    "",
    "def rq_set_pos_norm(pos_norm, gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos_gripper = norm_to_gripper(pos_norm)",
    "    rq_set_pos(pos_gripper, gripper_socket)",
    "end",
    "",
    "def rq_current_pos_norm(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    pos_gripper = rq_current_pos(gripper_socket)",
    "    pos_norm = gripper_to_norm(pos_gripper)",
    "    return pos_norm",
    "end",
    "",
    "def gripper_to_norm(value_gripper):",
    "    value_norm = (value_gripper / 255) * 100",
    "    return floor(value_norm)",
    "end",
    "",
    "def norm_to_gripper(value_norm):",
    "    value_gripper = (value_norm / 100) * 255",
    "    return ceil(value_gripper)",
    "end",
    "",
    "def rq_get_position():",
    "    return rq_current_pos_norm()",
    "end",
    "",
    "def rq_gripper_led_on(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,0, gripper_socket)",
    "end",
    "",
    "def rq_gripper_led_off(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,1, gripper_socket)",
    "    rq_set_var(LRD,0, gripper_socket)",
    "    rq_set_var(LBL,0, gripper_socket)",
    "    rq_set_var(LGN,0, gripper_socket)",
    "end",
    "",
    "def rq_gripper_led_force_red(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,1, gripper_socket)",
    "    rq_set_var(LRD,1, gripper_socket)",
    "    rq_set_var(LBL,0, gripper_socket)",
    "    rq_set_var(LGN,0, gripper_socket)",
    "end",
    "",
    "def rq_gripper_led_force_blue(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,1, gripper_socket)",
    "    rq_set_var(LRD,0, gripper_socket)",
    "    rq_set_var(LBL,1, gripper_socket)",
    "    rq_set_var(LGN,0, gripper_socket)",
    "end",
    "",
    "def rq_gripper_led_force_green(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,1, gripper_socket)",
    "    rq_set_var(LRD,0, gripper_socket)",
    "    rq_set_var(LBL,0, gripper_socket)",
    "    rq_set_var(LGN,1, gripper_socket)",
    "end",
    "",
    "def rq_gripper_led_force_purple(gripper_socket=" + QUOTATION + "1" + QUOTATION + "):",
    "    rq_set_var(LBP,1, gripper_socket)",
    "    rq_set_var(LRD,1, gripper_socket)",
    "    rq_set_var(LBL,1, gripper_socket)",
    "    rq_set_var(LGN,0, gripper_socket)",
    "end",
    "",
    "############################################",
    "# mm/inches functions",
    "############################################",
    "gripper_closed_norm = [100, 100, 100, 100]",
    "gripper_open_norm = [0, 0, 0, 0]",
    "gripper_closed_mm = [0, 0, 0, 0]",
    "gripper_open_mm = [50, 50, 50, 50]",
    "",
    "def rq_current_pos_mm(gripper_socket=1):",
    "    pos_gripper = rq_current_pos(gripper_socket)",
    "    pos_mm = gripper_to_mm(pos_gripper, gripper_socket)",
    "    return round_value_2_dec(pos_mm)",
    "end",
    "",
    "def rq_current_pos_inches(gripper_socket=1):",
    "    pos_gripper = rq_current_pos(gripper_socket)",
    "    pos_mm = gripper_to_mm(pos_gripper, gripper_socket)",
    "    pos_in = pos_mm / 25.4",
    "    return round_value_2_dec(pos_in)",
    "end",
    "",
    "def rq_move_mm(pos_mm, gripper_socket=1):",
    "    pos_gripper = mm_to_gripper(pos_mm, gripper_socket)",
    "    rq_move(pos_gripper, gripper_socket)",
    "end",
    "",
    "def rq_move_and_wait_mm(pos_mm, gripper_socket=1):",
    "    pos_gripper = mm_to_gripper(pos_mm, gripper_socket)",
    "    rq_move_and_wait(pos_gripper, gripper_socket)",
    "end",
    "",
    "def rq_move_inches(pos_in, gripper_socket=1):",
    "    pos_mm = pos_in * 25.4",
    "    rq_move_mm(pos_mm, gripper_socket)",
    "end",
    "",
    "def rq_move_and_wait_inches(pos_in, gripper_socket=1):",
    "    pos_mm = pos_in * 25.4",
    "    rq_move_and_wait_mm(pos_mm, gripper_socket)",
    "end",
    "",
    "def get_closed_norm(gripper_socket):",
    "    return gripper_closed_norm[gripper_socket - 1]",
    "end",
    "",
    "def get_open_norm(gripper_socket):",
    "    return gripper_open_norm[gripper_socket - 1]",
    "end",
    "",
    "def get_closed_mm(gripper_socket):",
    "    return gripper_closed_mm[gripper_socket - 1]",
    "end",
    "",
    "def get_open_mm(gripper_socket):",
    "    return gripper_open_mm[gripper_socket - 1]",
    "end",
    "",
    "def set_closed_norm(closed_norm, gripper_socket):",
    "    gripper_closed_norm[gripper_socket - 1] = closed_norm",
    "end",
    "",
    "def set_open_norm(open_norm, gripper_socket):",
    "    gripper_open_norm[gripper_socket - 1] = open_norm",
    "end",
    "",
    "def set_closed_mm(closed_mm, gripper_socket):",
    "    gripper_closed_mm[gripper_socket - 1] = closed_mm",
    "end",
    "",
    "def set_open_mm(open_mm, gripper_socket):",
    "    gripper_open_mm[gripper_socket - 1] = open_mm",
    "end",
    "",
    "def gripper_to_mm(value_gripper, gripper_socket):",
    "    closed_norm = get_closed_norm(gripper_socket)",
    "    open_norm = get_open_norm(gripper_socket)",
    "    closed_mm = get_closed_mm(gripper_socket)",
    "    open_mm = get_open_mm(gripper_socket)",
    "",
    "    value_norm = (value_gripper / 255) * 100",
    "",
    "    slope = (closed_mm - open_mm) / (closed_norm - open_norm)",
    "    value_mm = slope * (value_norm - closed_norm) + closed_mm",
    "",
    "    if (value_mm > open_mm):",
    "	value_mm_limited = open_mm",
    "    elif (value_mm < closed_mm):",
    "	value_mm_limited = closed_mm",
    "    else:",
    "	value_mm_limited = value_mm",
    "    end",
    "",
    "    return value_mm_limited",
    "end",
    "",
    "def mm_to_gripper(value_mm, gripper_socket):",
    "    closed_norm = get_closed_norm(gripper_socket)",
    "    open_norm = get_open_norm(gripper_socket)",
    "    closed_mm = get_closed_mm(gripper_socket)",
    "    open_mm = get_open_mm(gripper_socket)",
    "",
    "    slope = (closed_norm - open_norm) / (closed_mm - open_mm)",
    "    value_norm = (value_mm - closed_mm) * slope + closed_norm",
    "",
    "    value_gripper = value_norm * 255 / 100",
    "",
    "    if (value_gripper > 255):",
    "	value_gripper_limited = 255",
    "    elif (value_gripper < 0):",
    "	value_gripper_limited = 0",
    "    else:",
    "	value_gripper_limited = round_value(value_gripper)",
    "    end",
    "",
    "    return value_gripper_limited",
    "end",
    "",
    "def round_value(value):",
    "    value_mod = value % 1",
    "",
    "    if(value_mod < 0.5):",
    "	return floor(value)",
    "    else:",
    "	return ceil(value)",
    "    end",
    "end",
    "",
    "def round_value_2_dec(value):",
    "    value_x_100 = value * 100",
    "    value_x_100_rounded = round_value(value_x_100)",
    "    return value_x_100_rounded / 100",
    "end",
    "",
    "def clear_socket_buffer(gripper_socket=" + QUOTATION + "1" + QUOTATION + ", read_timeout = 0.1):",
    "  byte_in_buffer = socket_read_byte_list(1, gripper_socket, read_timeout)",
    "",
    "  while(byte_in_buffer[0] >= 1):",
    "      byte_in_buffer = socket_read_byte_list(1, gripper_socket, read_timeout)",
    "  end",
    "end",
    "",
    "def scale(value, rawRange, scaledRange):",
    "    def computeSlope(inputRange, outputRange):",
    "	outputRangeDelta = outputRange[1] - outputRange[0]",
    "	inputRangeDelta = inputRange[1] - inputRange[0]",
    "",
    "	if (inputRangeDelta == 0):",
    "	    return 0",
    "	else:",
    "	    return outputRangeDelta / inputRangeDelta",
    "	end",
    "    end",
    "",
    "    def computeIntercept(slope, inputRange, outputRange):",
    "	return outputRange[0] - (slope * inputRange[0])",
    "    end",
    "",
    "    def clipScaledValue(outputScaledValue, outputRange):",
    "	if (outputRange[0] < outputRange[1]):",
    "	    return clipWhenLowerLimitIsLessThanHigher(outputScaledValue, outputRange)",
    "	else:",
    "	    return clipWhenLowerLimitIsGreaterThanHigherLimit(outputScaledValue, outputRange)",
    "	end",
    "    end",
    "",
    "    def clipWhenLowerLimitIsGreaterThanHigherLimit(outputScaledValue, outputRange):",
    "	if (outputScaledValue < outputRange[1]):",
    "	    return outputRange[1]",
    "	elif (outputScaledValue > outputRange[0]):",
    "	    return outputRange[0]",
    "	else:",
    "	    return outputScaledValue",
    "	end",
    "    end",
    "",
    "    def clipWhenLowerLimitIsLessThanHigher(outputScaledValue, outputRange):",
    "	if (outputScaledValue < outputRange[0]):",
    "	    return outputRange[0]",
    "	elif (outputScaledValue > outputRange[1]):",
    "	    return outputRange[1]",
    "	else:",
    "	    return outputScaledValue",
    "	end",
    "    end",
    "",
    "    slope = computeSlope(rawRange, scaledRange)",
    "    intercept = computeIntercept(slope, rawRange, scaledRange)",
    "    scaledValue = slope * value + intercept",
    "    return clipScaledValue(scaledValue, scaledRange)",
    "end",
    "",
    "def limit(value, range):",
    "    return scale(value, range, range)",
    "end",
    "",
    "rq_obj_detect = 0",
    "#set_tool_voltage(24)",
    "#set_tool_communication(True, 115200, 0, 1, 1.5, 3.5)",
    "rq_wait_for_gripper_connected()",
    "rq_init_comm_if_connected(9, " + QUOTATION + "1" + QUOTATION + ")",
    "rq_init_comm_if_connected(2, " + QUOTATION + "2" + QUOTATION + ")",
    "rq_init_comm_if_connected(3, " + QUOTATION + "3" + QUOTATION + ")",
    "rq_init_comm_if_connected(4, " + QUOTATION + "4" + QUOTATION + ")",
    "rq_print_connected_grippers()",
    "connectivity_checked = [-1,-1,-1,-1]",
    "status_checked = [-1,-1,-1,-1]",
    "current_speed = [-1,-1,-1,-1]",
    "current_force = [-1,-1,-1,-1]",
    "set_closed_norm(100.0, 1)",
    "set_open_norm(0.0, 1)",
    "set_closed_mm(0.0, 1)",
    "set_open_mm(50.0, 1)",
    "set_closed_norm(100.0, 2)",
    "set_open_norm(0.0, 2)",
    "set_closed_mm(0.0, 2)",
    "set_open_mm(50.0, 2)",
    "set_closed_norm(100.0, 3)",
    "set_open_norm(0.0, 3)",
    "set_closed_mm(0.0, 3)",
    "set_open_mm(50.0, 3)",
    "set_closed_norm(100.0, 4)",
    "set_open_norm(0.0, 4)",
    "set_closed_mm(0.0, 4)",
    "set_open_mm(50.0, 4)",
    "",
])



from rtde_control import RTDEControlInterface as RTDEControl
from rtde_receive import RTDEReceiveInterface as RTDEReceive
import time

rtde_c = RTDEControl("127.0.0.1")
rtde_r = RTDEReceive("127.0.0.1")
init_q = rtde_r.getActualQ()

# Target in the robot base
new_q = init_q[:]
new_q[0] += 0.20

# Move asynchronously in joint space to new_q, we specify asynchronous behavior by setting the async parameter to
# 'True'. Try to set the async parameter to 'False' to observe a default synchronous movement, which cannot be stopped
# by the stopJ function due to the blocking behaviour.
rtde_c.moveJ(new_q, 1.05, 1.4, True)
time.sleep(0.2)
# Stop the movement before it reaches new_q
rtde_c.stopJ(0.5)

# Target in the Z-Axis of the TCP
target = rtde_r.getActualTCPPose()
target[2] += 0.10

# Move asynchronously in cartesian space to target, we specify asynchronous behavior by setting the async parameter to
# 'True'. Try to set the async parameter to 'False' to observe a default synchronous movement, which cannot be stopped
# by the stopL function due to the blocking behaviour.
rtde_c.moveL(target, 0.25, 0.5, True)
time.sleep(0.2)
# Stop the movement before it reaches target
rtde_c.stopL(0.5)

# Move back to initial joint configuration
rtde_c.moveJ(init_q)

# Stop the RTDE control script
rtde_c.stopScript()



"""Module to control Robotiq's grippers - tested with HAND-E"""

import socket
import threading
import time
from enum import Enum
from typing import Union, Tuple, OrderedDict

class RobotiqGripper:
    """
    Communicates with the gripper directly, via socket with string commands, leveraging string names for variables.
    """
    # WRITE VARIABLES (CAN ALSO READ)
    ACT = 'ACT'  # act : activate (1 while activated, can be reset to clear fault status)
    GTO = 'GTO'  # gto : go to (will perform go to with the actions set in pos, for, spe)
    ATR = 'ATR'  # atr : auto-release (emergency slow move)
    ADR = 'ADR'  # adr : auto-release direction (open(1) or close(0) during auto-release)
    FOR = 'FOR'  # for : force (0-255)
    SPE = 'SPE'  # spe : speed (0-255)
    POS = 'POS'  # pos : position (0-255), 0 = open
    # READ VARIABLES
    STA = 'STA'  # status (0 = is reset, 1 = activating, 3 = active)
    PRE = 'PRE'  # position request (echo of last commanded position)
    OBJ = 'OBJ'  # object detection (0 = moving, 1 = outer grip, 2 = inner grip, 3 = no object at rest)
    FLT = 'FLT'  # fault (0=ok, see manual for errors if not zero)

    ENCODING = 'UTF-8'  # ASCII and UTF-8 both seem to work

    class GripperStatus(Enum):
        """Gripper status reported by the gripper. The integer values have to match what the gripper sends."""
        RESET = 0
        ACTIVATING = 1
        # UNUSED = 2  # This value is currently not used by the gripper firmware
        ACTIVE = 3

    class ObjectStatus(Enum):
        """Object status reported by the gripper. The integer values have to match what the gripper sends."""
        MOVING = 0
        STOPPED_OUTER_OBJECT = 1
        STOPPED_INNER_OBJECT = 2
        AT_DEST = 3

    def __init__(self):
        """Constructor."""
        self.socket = None
        self.command_lock = threading.Lock()
        self._min_position = 0
        self._max_position = 255
        self._min_speed = 0
        self._max_speed = 255
        self._min_force = 0
        self._max_force = 255

    def connect(self, hostname: str, port: int, socket_timeout: float = 2.0) -> None:
        """Connects to a gripper at the given address.
        :param hostname: Hostname or ip.
        :param port: Port.
        :param socket_timeout: Timeout for blocking socket operations.
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((hostname, port))
        self.socket.settimeout(socket_timeout)

    def disconnect(self) -> None:
        """Closes the connection with the gripper."""
        self.socket.close()

    def _set_vars(self, var_dict: OrderedDict[str, Union[int, float]]):
        """Sends the appropriate command via socket to set the value of n variables, and waits for its 'ack' response.
        :param var_dict: Dictionary of variables to set (variable_name, value).
        :return: True on successful reception of ack, false if no ack was received, indicating the set may not
        have been effective.
        """
        # construct unique command
        cmd = "SET"
        for variable, value in var_dict.items():
            cmd += f" {variable} {str(value)}"
        cmd += '\n'  # new line is required for the command to finish
        # atomic commands send/rcv
        with self.command_lock:
            self.socket.sendall(cmd.encode(self.ENCODING))
            data = self.socket.recv(1024)
        return self._is_ack(data)

    def _set_var(self, variable: str, value: Union[int, float]):
        """Sends the appropriate command via socket to set the value of a variable, and waits for its 'ack' response.
        :param variable: Variable to set.
        :param value: Value to set for the variable.
        :return: True on successful reception of ack, false if no ack was received, indicating the set may not
        have been effective.
        """
        return self._set_vars(OrderedDict([(variable, value)]))

    def _get_var(self, variable: str):
        """Sends the appropriate command to retrieve the value of a variable from the gripper, blocking until the
        response is received or the socket times out.
        :param variable: Name of the variable to retrieve.
        :return: Value of the variable as integer.
        """
        # atomic commands send/rcv
        with self.command_lock:
            cmd = f"GET {variable}\n"
            self.socket.sendall(cmd.encode(self.ENCODING))
            data = self.socket.recv(1024)

        # expect data of the form 'VAR x', where VAR is an echo of the variable name, and X the value
        # note some special variables (like FLT) may send 2 bytes, instead of an integer. We assume integer here
        var_name, value_str = data.decode(self.ENCODING).split()
        if var_name != variable:
            raise ValueError(f"Unexpected response {data} ({data.decode(self.ENCODING)}): does not match '{variable}'")
        value = int(value_str)
        return value

    @staticmethod
    def _is_ack(data: str):
        return data == b'ack'

    def _reset(self):
        """
        Reset the gripper.
        The following code is executed in the corresponding script function
        def rq_reset(gripper_socket="1"):
            rq_set_var("ACT", 0, gripper_socket)
            rq_set_var("ATR", 0, gripper_socket)

            while(not rq_get_var("ACT", 1, gripper_socket) == 0 or not rq_get_var("STA", 1, gripper_socket) == 0):
                rq_set_var("ACT", 0, gripper_socket)
                rq_set_var("ATR", 0, gripper_socket)
                sync()
            end

            sleep(0.5)
        end
        """
        self._set_var(self.ACT, 0)
        self._set_var(self.ATR, 0)
        while (not self._get_var(self.ACT) == 0 or not self._get_var(self.STA) == 0):
            self._set_var(self.ACT, 0)
            self._set_var(self.ATR, 0)
        time.sleep(0.5)


    def activate(self, auto_calibrate: bool = True):
        """Resets the activation flag in the gripper, and sets it back to one, clearing previous fault flags.
        :param auto_calibrate: Whether to calibrate the minimum and maximum positions based on actual motion.
        The following code is executed in the corresponding script function
        def rq_activate(gripper_socket="1"):
            if (not rq_is_gripper_activated(gripper_socket)):
                rq_reset(gripper_socket)

                while(not rq_get_var("ACT", 1, gripper_socket) == 0 or not rq_get_var("STA", 1, gripper_socket) == 0):
                    rq_reset(gripper_socket)
                    sync()
                end

                rq_set_var("ACT",1, gripper_socket)
            end
        end
        def rq_activate_and_wait(gripper_socket="1"):
            if (not rq_is_gripper_activated(gripper_socket)):
                rq_activate(gripper_socket)
                sleep(1.0)

                while(not rq_get_var("ACT", 1, gripper_socket) == 1 or not rq_get_var("STA", 1, gripper_socket) == 3):
                    sleep(0.1)
                end

                sleep(0.5)
            end
        end
        """
        if not self.is_active():
            self._reset()
            while (not self._get_var(self.ACT) == 0 or not self._get_var(self.STA) == 0):
                time.sleep(0.01)

            self._set_var(self.ACT, 1)
            time.sleep(1.0)
            while (not self._get_var(self.ACT) == 1 or not self._get_var(self.STA) == 3):
                time.sleep(0.01)

        # auto-calibrate position range if desired
        if auto_calibrate:
            self.auto_calibrate()

    def is_active(self):
        """Returns whether the gripper is active."""
        status = self._get_var(self.STA)
        return RobotiqGripper.GripperStatus(status) == RobotiqGripper.GripperStatus.ACTIVE

    def get_min_position(self) -> int:
        """Returns the minimum position the gripper can reach (open position)."""
        return self._min_position

    def get_max_position(self) -> int:
        """Returns the maximum position the gripper can reach (closed position)."""
        return self._max_position

    def get_open_position(self) -> int:
        """Returns what is considered the open position for gripper (minimum position value)."""
        return self.get_min_position()

    def get_closed_position(self) -> int:
        """Returns what is considered the closed position for gripper (maximum position value)."""
        return self.get_max_position()

    def is_open(self):
        """Returns whether the current position is considered as being fully open."""
        return self.get_current_position() <= self.get_open_position()

    def is_closed(self):
        """Returns whether the current position is considered as being fully closed."""
        return self.get_current_position() >= self.get_closed_position()

    def get_current_position(self) -> int:
        """Returns the current position as returned by the physical hardware."""
        return self._get_var(self.POS)

    def auto_calibrate(self, log: bool = True) -> None:
        """Attempts to calibrate the open and closed positions, by slowly closing and opening the gripper.
        :param log: Whether to print the results to log.
        """
        # first try to open in case we are holding an object
        (position, status) = self.move_and_wait_for_pos(self.get_open_position(), 64, 1)
        if RobotiqGripper.ObjectStatus(status) != RobotiqGripper.ObjectStatus.AT_DEST:
            raise RuntimeError(f"Calibration failed opening to start: {str(status)}")

        # try to close as far as possible, and record the number
        (position, status) = self.move_and_wait_for_pos(self.get_closed_position(), 64, 1)
        if RobotiqGripper.ObjectStatus(status) != RobotiqGripper.ObjectStatus.AT_DEST:
            raise RuntimeError(f"Calibration failed because of an object: {str(status)}")
        assert position <= self._max_position
        self._max_position = position

        # try to open as far as possible, and record the number
        (position, status) = self.move_and_wait_for_pos(self.get_open_position(), 64, 1)
        if RobotiqGripper.ObjectStatus(status) != RobotiqGripper.ObjectStatus.AT_DEST:
            raise RuntimeError(f"Calibration failed because of an object: {str(status)}")
        assert position >= self._min_position
        self._min_position = position

        if log:
            print(f"Gripper auto-calibrated to [{self.get_min_position()}, {self.get_max_position()}]")

    def move(self, position: int, speed: int, force: int) -> Tuple[bool, int]:
        """Sends commands to start moving towards the given position, with the specified speed and force.
        :param position: Position to move to [min_position, max_position]
        :param speed: Speed to move at [min_speed, max_speed]
        :param force: Force to use [min_force, max_force]
        :return: A tuple with a bool indicating whether the action it was successfully sent, and an integer with
        the actual position that was requested, after being adjusted to the min/max calibrated range.
        """

        def clip_val(min_val, val, max_val):
            return max(min_val, min(val, max_val))

        clip_pos = clip_val(self._min_position, position, self._max_position)
        clip_spe = clip_val(self._min_speed, speed, self._max_speed)
        clip_for = clip_val(self._min_force, force, self._max_force)

        # moves to the given position with the given speed and force
        var_dict = OrderedDict([(self.POS, clip_pos), (self.SPE, clip_spe), (self.FOR, clip_for), (self.GTO, 1)])
        return self._set_vars(var_dict), clip_pos

    def move_and_wait_for_pos(self, position: int, speed: int, force: int) -> Tuple[int, ObjectStatus]:  # noqa
        """Sends commands to start moving towards the given position, with the specified speed and force, and
        then waits for the move to complete.
        :param position: Position to move to [min_position, max_position]
        :param speed: Speed to move at [min_speed, max_speed]
        :param force: Force to use [min_force, max_force]
        :return: A tuple with an integer representing the last position returned by the gripper after it notified
        that the move had completed, a status indicating how the move ended (see ObjectStatus enum for details). Note
        that it is possible that the position was not reached, if an object was detected during motion.
        """
        set_ok, cmd_pos = self.move(position, speed, force)
        if not set_ok:
            raise RuntimeError("Failed to set variables for move.")

        # wait until the gripper acknowledges that it will try to go to the requested position
        while self._get_var(self.PRE) != cmd_pos:
            time.sleep(0.001)

        # wait until not moving
        cur_obj = self._get_var(self.OBJ)
        while RobotiqGripper.ObjectStatus(cur_obj) == RobotiqGripper.ObjectStatus.MOVING:
            cur_obj = self._get_var(self.OBJ)

        # report the actual position and the object status
        final_pos = self._get_var(self.POS)
        final_obj = cur_obj
        return final_pos, RobotiqGripper.ObjectStatus(final_obj)


from conans import ConanFile
from conans.tools import os_info

class UR_Rtde(ConanFile):
    settings = 'os', 'compiler', 'build_type', 'arch'
    generators = 'cmake'

    def requirements(self):
        self.requires('boost/1.75.0')

    def imports(self):
        self.copy('*.dll', dst='bin', src='bin')



import rtde_control
from robotiq_preamble import ROBOTIQ_PREAMBLE
import time


class RobotiqGripper(object):
    """ 
    RobotiqGripper is a class for controlling a robotiq gripper using the
    ur_rtde robot interface. 
      
    Attributes: 
        rtde_c (rtde_control.RTDEControlInterface): The interface to use for the communication
    """
    def __init__(self, rtde_c): 
        """ 
        The constructor for RobotiqGripper class. 
  
        Parameters: 
           rtde_c (rtde_control.RTDEControlInterface): The interface to use for the communication
        """
        self.rtde_c = rtde_c

    def call(self, script_name, script_function):
        return self.rtde_c.sendCustomScriptFunction(
            "ROBOTIQ_" + script_name,
            ROBOTIQ_PREAMBLE + script_function
        )

    def activate(self):
        """ 
        Activates the gripper. Currently the activation will take 5 seconds.
           
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        ret = self.call("ACTIVATE", "rq_activate()")
        time.sleep(5)  # HACK
        return ret

    def set_speed(self, speed):
        """ 
        Set the speed of the gripper. 
  
        Parameters: 
            speed (int): speed as a percentage [0-100]
          
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        return self.call("SET_SPEED", "rq_set_speed_norm(" + str(speed) + ")")

    def set_force(self, force):
        """ 
        Set the force of the gripper. 
  
        Parameters: 
            force (int): force as a percentage [0-100]
          
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        return self.call("SET_FORCE", "rq_set_force_norm(" + str(force) + ")")

    def move(self, pos_in_mm):
        """ 
        Move the gripper to a specified position in (mm).
  
        Parameters: 
            pos_in_mm (int): position in millimeters.
          
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        return self.call("MOVE", "rq_move_and_wait_mm(" + str(pos_in_mm) + ")")

    def open(self):
        """ 
        Open the gripper.
           
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        return self.call("OPEN", "rq_open_and_wait()")

    def close(self):
        """ 
        Close the gripper.
           
        Returns: 
            True if the command succeeded, otherwise it returns False
        """
        return self.call("CLOSE", "rq_close_and_wait()")
